package sample.handlers;

import javax.inject.Inject;
import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MBasicFactory;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MPartStack;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.ESelectionService;

import sample.product.Item;

public class OpenTreeNodeHandler {

    @Inject
    private EPartService partService;
    @Inject
    private EModelService modelService;
    @Inject
    private MApplication app;
    @Inject
    private ESelectionService selectionService;

    @Execute
    public void openNode(@Named(IServiceConstants.ACTIVE_SELECTION) Item selectedItem) {

        System.out.println("OpenTreeNodeHandler - Handler invoked with selected item " + selectedItem);
        String partId = createEditorPartIdFor(selectedItem);
        MPart editorPart = partService.findPart(partId);
        if (editorPart == null) {
            System.out.println("OpenTreeNodeHandler - No editor found with id=" + partId);
            editorPart = createEditorFor(selectedItem);

            MPartStack editorsStack = (MPartStack) modelService.find("sample.part.stack.editors", app);
            editorsStack.getChildren().add(editorPart);
            System.out.println("OpenTreeNodeHandler - Editor with id=" + partId + " created and added to stack");
        } else {
            System.out.println("OpenTreeNodeHandler - Editor with id=" + partId + " already exists");
        }

        partService.activate(editorPart);
        System.out.println("OpenTreeNodeHandler - Editor with id=" + partId + " activated");
    }

    private String createEditorPartIdFor(Item selectedItem) {
        return "sample.editor." + selectedItem.toString();
    }

    private MPart createEditorFor(Item selectedItem) {
        MPart editorPart = MBasicFactory.INSTANCE.createPart();
        editorPart.setElementId(createEditorPartIdFor(selectedItem));
        String shortName = selectedItem.toString();
        editorPart.setLabel(shortName);
        editorPart.setTooltip(shortName);
        editorPart.setCloseable(true);
        editorPart.setContributionURI("bundleclass://sample/sample.product.ItemEditorPart");
        return editorPart;
    }

}
